// File: SAXValidator.java

import java.io.File;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.Locator;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

public class SAXValidator {
  static final String SCHEMA_LANGUAGE =
    "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
  static final String SCHEMA_SOURCE =
    "http://java.sun.com/xml/jaxp/properties/schemaSource";
  static final String W3C_SCHEMA = "http://www.w3.org/2001/XMLSchema";

  public static void main(String args[]) {
    if (args.length != 2) {
      System.out.println(
        "Usage: java SAXValidator xmlfile.xml schemafile.xsd");
      System.exit(1);
    }
    DefaultHandler handler = new SAXValidatorHandler();
    SAXParserFactory factory = SAXParserFactory.newInstance();
    factory.setValidating(true);
    factory.setNamespaceAware(true);
    try {
      SAXParser parser = factory.newSAXParser();
      parser.setProperty(SCHEMA_LANGUAGE, W3C_SCHEMA);
      parser.setProperty(SCHEMA_SOURCE, args[1]);
      parser.parse(new File(args[0]), handler);
    }
    catch (Exception e) {
      System.err.println(e);
    }
  }
}

class SAXValidatorHandler extends DefaultHandler {
  private Locator locator;
  public void setDocumentLocator(Locator loc) {
    locator = loc;
  }
  public void error(SAXParseException e) {
    System.err.println("Validation error on line " +
      locator.getLineNumber() + ":\n" + e.getMessage());
  }
  public void fatalError(SAXParseException e) {
    System.err.println("Well-formedness error on line " +
      locator.getLineNumber() + ":\n" + e.getMessage());
  }
}

